#! /usr/bin/perl

#############################################################################
# slashpluck.pl
#
# $Id: slashpluck.pl,v 1.13 2003/02/23 18:21:27 jday Exp $
# 
# Copyright (C) 2002 by Jason Day <jasonday@worldnet.att.net>
#
# Fetches headlines and stories from Slashdot and formats them in a friendly way
# for Plucker (http://www.plkr.org/).  The headlines and articles are stored as
# html files in a directory accessible by plucker.
#
# The timestamp of the headlines file is checked first, so that fetching occurs
# no more than once every 30 minutes (this was a guideline specified in the
# slashdot FAQ, though the most recent FAQ doesn't seem to have it).
#
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#############################################################################

use strict;
use vars qw($directory $threshold $headlinesfile $tmpfile $article_stylesheet
            $VERSION $DEBUG $LOCAL_RSS);
use LWP::UserAgent;

#############################################################################
# Configuration section
#############################################################################

# The directory where the html files are stored.
$directory = "$ENV{HOME}/.slashpluck";

# Article threshold.  Only articles with this score or higher will be
# retrieved.  I suggest using 3 or higher, unless you have lots of room
# on your PDA.
$threshold = 3;

# Article stylesheet. This is the name of the XSL stylesheet used to transform
# the articles and comments.
$article_stylesheet = "$directory/article.xsl";

#############################################################################
# End of configuration section
#############################################################################

# Turn off buffered output.
$|++;

# SlashPluck version number.
$VERSION = "0.40";

# Uncomment this to get debugging information.
#$DEBUG++;

#$LOCAL_RSS++;

# Filename for the headlines.  There's not really any reason to change this,
# but it could be anything you want, as long as it's a legal filename.
$headlinesfile = "$directory/headlines.html";

# Temorary file for processing articles.
$tmpfile = "$directory/article.html";

#############################################################################

sub fetch() {
    my $update_time = localtime(time);
    my $ua = LWP::UserAgent->new;
    $ua->agent ("SlashPluck/$VERSION");

    open (OUTFILE, ">$headlinesfile");
    print OUTFILE <<EOF;
<html>
<head>
<title>Slashdot Headlines</title>
</head>
<body bgcolor="white">
<center>
<img src="topicslashdot.gif" width="100" height="34" border="0" alt="Slashdot.org">
<br>
<b>News for Nerds: Stuff that Matters.</b>
<p>
Last updated: $update_time
</center>
EOF

    my $request;
    my $response;

    if ($LOCAL_RSS) {
        print "Using local slashdot.rss\n";
    }
    else {
        print "Fetching slashdot.rss\n";
        $request = HTTP::Request->new ('GET', "http://slashdot.org/slashdot.rss");
        $response = $ua->request ($request, "$directory/slashdot.rss");
        if ($response->is_error) {
            die "Error fetching slashdot.rss";
        }
    }

    open (XMLFILE, "$directory/slashdot.rss")
        or die "Error reading slashdot.rss";

    my $skip = 1;
    my $artnum = 0;
    my ($title, $link, $desc);

    while (<XMLFILE>) {
        if (/<item rdf:about="([^"]*)">/) {
            $skip = 0;
            $link = "$1&light=1&threshold=$threshold";
            $artnum++;
            print "Fetching article: $link\n";
	    $request = HTTP::Request->new ('GET', "$link");
            $response = $ua->request ($request, "$tmpfile");
	    if ($response->is_error) {
                die "Error fetching article";
	    }

            system "xsltproc --html -o $directory/$artnum.html $article_stylesheet $tmpfile";

            next;
        }
        next if ($skip);

        if (/<title>([^<]*)<\/title>/) {
            $title = $1;
            next;
        }

        if (/<description>([^<]*)<\/description>/) {
            $desc = $1;
            $desc =~ s/\&lt;p\&gt;\&lt;a href.*\&gt;&lt;\/a\&gt;\&lt;\/p\&gt;//;
            print OUTFILE <<EOF;
<p><b><a href="$artnum.html">$title</a></b><br>
$desc<br>
<a href="$artnum.html">Read More</a></p>
EOF

            $skip = 1;
        }
    }
    close (XMLFILE);
    unlink "$tmpfile";
    
    print OUTFILE <<EOF;
</body>
</html>
EOF
    
    close (OUTFILE);
}

#############################################################################

# Create the slashpluck directory if necessary
unless (-d $directory) {
    print "Creating directory $directory\n";
    mkdir $directory, 0755 or die "Error creating directory $directory\n";
}

# Make sure we can write to the directory
die "Cannot write to directory $directory. Check your configuration.\n"
    unless (-w $directory);

# Check the modification time of the headlines file.  If it's more than
# 30 minutes then refetch the headlines and articles from slashdot.
if (-e $headlinesfile) {
    my $mtime = (stat ($headlinesfile))[9];
    my $diff = time - $mtime;
    if ($diff >= 1800) {
        &fetch();
    }
    else {
        print "$headlinesfile was last updated $diff seconds ago, fetch skipped.\n";
    }
}
else {
    # First-time run, so go fetch
    &fetch();
}
